@@ -152,15 +152,23 @@ ifa_send_notify(struct proto *p, unsigned c, struct ifa *a)
 }
 
 static void
-ifa_notify_change(unsigned c, struct ifa *a)
+ifa_notify_change_dep(unsigned c, struct ifa *a)
 {
   struct proto *p;
 
   DBG("IFA change notification (%x) for %s:%I\n", c, a->iface->name, a->ip);
+
   WALK_LIST(p, active_proto_list)
     ifa_send_notify(p, c, a);
 }
 
+static inline void
+ifa_notify_change(unsigned c, struct ifa *a)
+{
+  neigh_ifa_update(a);
+  ifa_notify_change_dep(c, a);
+}
+
 static inline void
 if_send_notify(struct proto *p, unsigned c, struct iface *i)
 {
@@ -197,11 +205,12 @@ if_notify_change(unsigned c, struct iface *i)
 
   if (c & IF_CHANGE_UP)
     neigh_if_up(i);
+
   if (c & IF_CHANGE_DOWN)
     WALK_LIST(a, i->addrs)
       {
 	a->flags = (i->flags & ~IA_FLAGS) | (a->flags & IA_FLAGS);
-	ifa_notify_change(IF_CHANGE_DOWN, a);
+	ifa_notify_change_dep(IF_CHANGE_DOWN, a);
       }
 
   WALK_LIST(p, active_proto_list)
@@ -211,7 +220,7 @@ if_notify_change(unsigned c, struct iface *i)
     WALK_LIST(a, i->addrs)
       {
 	a->flags = (i->flags & ~IA_FLAGS) | (a->flags & IA_FLAGS);
-	ifa_notify_change(IF_CHANGE_UP, a);
+	ifa_notify_change_dep(IF_CHANGE_UP, a);
       }
 
   if ((c & (IF_CHANGE_UP | IF_CHANGE_DOWN | IF_CHANGE_LINK)) == IF_CHANGE_LINK)
@@ -415,6 +424,24 @@ if_find_by_name(char *name)
   return NULL;
 }
 
+struct iface *
+if_get_by_name(char *name)
+{
+  struct iface *i;
+
+  if (i = if_find_by_name(name))
+    return i;
+
+  /* No active iface, create a dummy */
+  i = mb_allocz(if_pool, sizeof(struct iface));
+  strncpy(i->name, name, sizeof(i->name)-1);
+  i->flags = IF_SHUTDOWN;
+  init_list(&i->addrs);
+  init_list(&i->neighbors);
+  add_tail(&iface_list, &i->n);
+  return i;
+}
+
 struct ifa *kif_choose_primary(struct iface *i);
 
 static int
