@@ -200,6 +200,11 @@ typedef struct rte {
       u32 tag;				/* External route tag */
       u32 router_id;			/* Router that originated this route */
     } ospf;
+#endif
+#ifdef CONFIG_BGP
+    struct {
+      u8 suppressed;			/* Used for deterministic MED comparison */
+    } bgp;
 #endif
     struct {				/* Routes generated by krt sync (both temporary and inherited ones) */
       s8 src;				/* Alleged route source (see krt.h) */
