@@ -11,7 +11,7 @@
 #define _BIRD_OSPF_DBDES_H_
 
 void ospf_dbdes_send(struct ospf_neighbor *n, int next);
-void ospf_dbdes_receive(struct ospf_dbdes_packet *ps,
-			struct ospf_iface *ifa, struct ospf_neighbor *n);
+void ospf_dbdes_receive(struct ospf_packet *ps, struct ospf_iface *ifa,
+			struct ospf_neighbor *n);
 
 #endif /* _BIRD_OSPF_DBDES_H_ */
