@@ -34,13 +34,9 @@ struct krt_set_params {
 struct krt_set_status {
 };
 
-struct krt_if_params {
-};
-
-struct krt_if_status {
-};
-
 static inline int krt_set_params_same(struct krt_set_params *o UNUSED, struct krt_set_params *n UNUSED) { return 1; }
+static inline void krt_set_copy_params(struct krt_set_params *d UNUSED, struct krt_set_params *s UNUSED) { }
+
 void krt_read_msg(struct proto *p, struct ks_msg *msg, int scan);
 
 #endif
